import { DeviceType } from '@ucap-webmessenger/core';
import {
  APIRequest,
  APIResponse,
  APIEncoder,
  APIDecoder,
  ParameterUtil,
  StatusCode,
  JsonAnalization
} from '@ucap-webmessenger/api';

export interface TransMassTalkDownloadRequest extends APIRequest {
  userSeq: number;
  deviceType: DeviceType;
  token: string;
  eventTransSeq?: string;
}

export interface TransMassTalkDownloadResponse extends APIResponse {
  content?: string;
  userName?: string;
  regDate?: string;
  returnJson?: any;
}

const transMassTalkDownloadEncodeMap = {
  userSeq: 'p_user_seq',
  deviceType: 'p_device_type',
  token: 'p_token',
  eventTransSeq: 'p_event_mass_seq'
};

export const encodeTransMassTalkDownload: APIEncoder<TransMassTalkDownloadRequest> = (
  req: TransMassTalkDownloadRequest
) => {
  return ParameterUtil.encode(transMassTalkDownloadEncodeMap, req);
};

export const decodeTransMassTalkDownload: APIDecoder<TransMassTalkDownloadResponse> = (
  res: any
) => {
  try {
    const json = JsonAnalization.receiveAnalization(res);
    return {
      statusCode: json.StatusCode,
      content: json.Content,
      userName: json.UserName,
      regDate: json.RegDate,
      returnJson: res
    } as TransMassTalkDownloadResponse;
  } catch (e) {
    return {
      statusCode: StatusCode.Fail,
      errorMessage: e
    } as TransMassTalkDownloadResponse;
  }
};
